@@ -1,19 +1,12 @@
-try:
-    import ujson as json
-except ImportError:
-    import json
 import os
 import sys
 from fnmatch import fnmatch
 import logging
-import re
-import copy
 
 import requests.sessions
 from requests.adapters import HTTPAdapter
 from requests.exceptions import HTTPError
 from requests import Response
-from jsonpatch import make_patch
 from clint.textui import progress
 import six
 
@@ -24,7 +17,7 @@ log = logging.getLogger(__name__)
 
 
 # Item class
-#_________________________________________________________________________________________
+# ________________________________________________________________________________________
 class Item(object):
     """This class represents an archive.org item. You can use this
     class to access item metadata::
@@ -53,8 +46,9 @@ class Item(object):
 
     """
     # init()
-    #_____________________________________________________________________________________
-    def __init__(self, identifier, metadata_timeout=None, config=None, max_retries=1):
+    # ____________________________________________________________________________________
+    def __init__(self, identifier, metadata_timeout=None, config=None, max_retries=1,
+                 archive_session=None):
         """
         :type identifier: str
         :param identifier: The globally unique Archive.org identifier
@@ -71,8 +65,13 @@ class Item(object):
         :param max_retries: (optional) Maximum number of times to request
                             a website if the connection drops. (default: 1)
 
+        :type archive_session: :class:`ArchiveSession <ArchiveSession>` 
+        :param archive_session: An :class:`ArchiveSession <ArchiveSession>`
+                                object can be provided via the `archive_session`
+                                parameter.
+
         """
-        self.session = session.ArchiveSession(config)
+        self.session = archive_session if archive_session else session.get_session(config)
         self.protocol = 'https:' if self.session.secure else 'http:'
         self.http_session = requests.sessions.Session()
         max_retries_adapter = HTTPAdapter(max_retries=max_retries)
@@ -99,13 +98,13 @@ class Item(object):
         self.exists = False if self._json == {} else True
 
     # __repr__()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def __repr__(self):
         return ('Item(identifier={identifier!r}, '
                 'exists={exists!r})'.format(**self.__dict__))
 
     # get_metadata()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def get_metadata(self, metadata_timeout=None):
         """Get an item's metadata from the `Metadata API
         <http://blog.archive.org/2013/07/04/metadata-api/>`__
@@ -131,7 +130,7 @@ class Item(object):
         return metadata
 
     # iter_files()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def iter_files(self):
         """Generator for iterating over files in an item.
 
@@ -145,7 +144,7 @@ class Item(object):
             yield file
 
     # file()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def get_file(self, file_name):
         """Get a :class:`File <File>` object for the named file.
 
@@ -158,7 +157,7 @@ class Item(object):
                 return f
 
     # get_files()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def get_files(self, files=None, source=None, formats=None, glob_pattern=None):
         files = [] if not files else files
         source = [] if not source else source
@@ -179,12 +178,17 @@ class Item(object):
             elif f.format in formats:
                 file_objects.append(f)
             elif glob_pattern:
-                if fnmatch(f.name, glob_pattern):
-                    file_objects.append(f)
+                # Support for | operator.
+                patterns = glob_pattern.split('|')
+                if not isinstance(patterns, list):
+                    patterns = [patterns]
+                for p in patterns:
+                    if fnmatch(f.name, p):
+                        file_objects.append(f)
         return file_objects
 
     # download()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def download(self, concurrent=False, source=None, formats=None, glob_pattern=None,
                  dry_run=False, verbose=False, ignore_existing=False):
         """Download the entire item into the current working directory.
@@ -256,7 +260,7 @@ class Item(object):
         return True
 
     # modify_metadata()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def modify_metadata(self, metadata, target=None, append=False, priority=None,
                         access_key=None, secret_key=None, debug=False):
         """Modify the metadata of an existing item on Archive.org.
@@ -298,6 +302,7 @@ class Item(object):
             priority=priority,
             access_key=access_key,
             secret_key=secret_key,
+            append=append,
         )
         if debug:
             return request
@@ -307,7 +312,7 @@ class Item(object):
         return resp
 
     # upload_file()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def upload_file(self, body, key=None, metadata={}, headers={},
                     access_key=None, secret_key=None, queue_derive=True,
                     ignore_preexisting_bucket=False, verbose=False, verify=True,
@@ -406,7 +411,7 @@ class Item(object):
 
         # require the Content-MD5 header when delete is True.
         if verify or delete:
-            headers['Content-MD5'] = md5_sum 
+            headers['Content-MD5'] = md5_sum
         if verbose:
             try:
                 chunk_size = 1048576
@@ -429,6 +434,7 @@ class Item(object):
             metadata=metadata,
             access_key=access_key,
             secret_key=secret_key,
+            queue_derive=queue_derive,
             **kwargs
         )
 
@@ -452,9 +458,9 @@ class Item(object):
                 error_msg = (' error uploading {0} to {1}, '
                              '{2}'.format(key, self.identifier, exc))
                 if response.status_code == 400 and \
-                    any('\n' in str(v) for (k, v) in request.headers.items()):
-                        error_msg += (' - Check your metadata and header values for '
-                                      'newline charchters (i.e. "\\n").')
+                        any('\n' in str(v) for (k, v) in request.headers.items()):
+                    error_msg += (' - Check your metadata and header values for '
+                                  'newline charchters (i.e. "\\n").')
                 log.error(error_msg)
                 if verbose:
                     sys.stderr.write(error_msg + '\n')
@@ -462,7 +468,7 @@ class Item(object):
                 raise type(exc)(error_msg)
 
     # upload()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def upload(self, files, **kwargs):
         """Upload files to an item. The item will be created if it
         does not exist.
@@ -520,7 +526,7 @@ class Item(object):
 
 
 # File class
-#_________________________________________________________________________________________
+# ________________________________________________________________________________________
 class File(object):
     """This class represents a file in an archive.org item. You
     can use this class to access the file metadata::
@@ -548,7 +554,7 @@ class File(object):
 
     """
     # init()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def __init__(self, item, name):
         """
         :type item: Item
@@ -573,11 +579,11 @@ class File(object):
         for key in _file:
             setattr(self, key, _file[key])
         base_url = '{protocol}//archive.org/download/{identifier}'.format(**item.__dict__)
-        self.url = '{base_url}/{name}'.format(base_url=base_url, 
+        self.url = '{base_url}/{name}'.format(base_url=base_url,
                                               name=name.encode('utf-8'))
 
     # __repr__()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def __repr__(self):
         return ('File(identifier={identifier!r}, '
                 'filename={name!r}, '
@@ -586,7 +592,7 @@ class File(object):
                 'format={format!r})'.format(**self.__dict__))
 
     # download()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def download(self, file_path=None, ignore_existing=False):
         """Download the file into the current working directory.
 
@@ -616,7 +622,7 @@ class File(object):
                     f.flush()
 
     # delete()
-    #_____________________________________________________________________________________
+    # ____________________________________________________________________________________
     def delete(self, debug=False, verbose=False, cascade_delete=False, access_key=None,
                secret_key=None):
         """Delete a file from the Archive. Note: Some files -- such as
@@ -633,7 +639,8 @@ class File(object):
         :param cascade_delete: Also deletes files derived from the file,
                                and files the file was derived from.
         """
-        url = 'http://s3.us.archive.org/{0}/{1}'.format(self.identifier, self.name)
+        url = 'http://s3.us.archive.org/{0}/{1}'.format(self.identifier,
+                                                        self.name.encode('utf-8'))
         access_key = self._item.session.access_key if not access_key else access_key
         secret_key = self._item.session.secret_key if not secret_key else secret_key
         request = iarequest.S3Request(
@@ -647,6 +654,6 @@ class File(object):
             return request
         else:
             if verbose:
-                sys.stdout.write(' deleting file: {0}\n'.format(self.name))
+                sys.stdout.write(' deleting file: {0}\n'.format(self.name.encode('utf-8')))
             prepared_request = request.prepare()
             return self._item.http_session.send(prepared_request)
